@@ -6,7 +6,7 @@
 #include <cstdint>
 #include <thread>
 
-static volatile int *Sink;
+static int * volatile Sink;
 
 extern "C" int LLVMFuzzerTestOneInput(const uint8_t *Data, size_t Size) {
   if (Size == 0) return 0;
